#!/usr/bin/env python

import glob
import os
import socket

from resource_management.core.logger import Logger
from resource_management.core.resources.system import Execute
from resource_management.libraries.script.script import Script


HADOOP_FS = "HADOOP_USER_NAME=hdfs hadoop fs "


class JuiceFSClient(Script):
    def install(self, env):
        import params

        download_url = params.download_url
        local_download_dir = params.local_download_dir
        hdfs_download_dir = params.share_download_dir
        install_dirs = params.install_dirs.split(",")
        cache_dirs = params.cache_dirs

        server_hostname = params.ambari_server_hostname
        self._downlod_jar(download_url, local_download_dir, hdfs_download_dir, server_hostname)

        self._ln_jars(local_download_dir, install_dirs)
        self._make_cache_dir(cache_dirs)

    def _run(self, cmd):
        Logger.info("Executing: %s" % cmd)
        if os.system(cmd) == 0:
            return True
        return False

    def _downlod_jar(self, download_url, local_download_dir, hdfs_download_dir, server_hostname):
        basename = os.path.basename(download_url)
        local_path = os.path.join(local_download_dir, basename)
        tmp_local_path = os.path.join(local_download_dir, basename + ".tmp")
        hdfs_path = os.path.join(hdfs_download_dir, basename)

        if os.path.exists(tmp_local_path):
            os.remove(tmp_local_path)

        if socket.gethostbyname(server_hostname) == socket.gethostbyname(socket.gethostname()):
            if not self._run("%s -test -e %s" % (HADOOP_FS, hdfs_path)):
                self._download_from_url(download_url, tmp_local_path)
                self._copy_to_hdfs(tmp_local_path, hdfs_path)

        if not self._download_from_hdfs(hdfs_path, tmp_local_path):
            self._download_from_url(download_url, tmp_local_path)

        if os.path.exists(tmp_local_path) and self._md5(tmp_local_path) == self._md5(local_path):
            os.remove(tmp_local_path)
            return
        else:
            try:
                os.remove(local_path)
            except OSError:
                pass
            os.rename(tmp_local_path, local_path)

        # make symlink
        link = os.path.join(local_download_dir, "juicefs-hadoop.jar")
        if os.path.exists(link) or os.path.islink(link):
            os.remove(link)
        os.symlink(local_path, link)

    def _download_from_url(self, download_url, tmp_local_path):
        Execute("curl -o %s %s" % (tmp_local_path, download_url))
        os.chmod(tmp_local_path, 0o644)

    def _copy_to_hdfs(self, local_path, hdfs_path):
        # check
        dst_dir = os.path.dirname(hdfs_path)
        if self._run("%s -test -f %s" % (HADOOP_FS, dst_dir)):
            Logger.error("%s is a file" % dst_dir)
        if not self._run("%s -test -e %s" % (HADOOP_FS, dst_dir)):
            self._run("%s -mkdir -p %s" % (HADOOP_FS, dst_dir))

        if not self._run("%s -put -f %s %s" % (HADOOP_FS, local_path, hdfs_path)):
            Logger.error("failed to put %s to hdfs %s" % (local_path, hdfs_path))

    def _download_from_hdfs(self, hdfs_path, tmp_local_path):
        count = 30
        sleep = 3

        import time

        while count > 0:
            if os.path.exists(tmp_local_path):
                os.remove(tmp_local_path)
            Logger.info("Downloading from hdfs %s " % hdfs_path)
            if self._run("%s -get -p %s %s" % (HADOOP_FS, hdfs_path, tmp_local_path)):
                os.chmod(tmp_local_path, 0o644)
                return True
            time.sleep(sleep)
            count = count - 1
        Logger.info("failed to download from %s, switch to download from url" % hdfs_path)
        return False

    def _md5(self, f):
        if not os.path.exists(f):
            return None
        import hashlib

        hash_md5 = hashlib.md5()
        with open(f, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()

    def _ln_jars(self, download_dir, install_dirs):
        link = os.path.join(download_dir, "juicefs-hadoop.jar")
        import params

        java_home = params.java_home

        # auto create hive auxlib
        if os.path.exists("/usr/hdp/current/hive-client"):
            hive_aux_dir = os.path.join("/usr/hdp/current/hive-client/auxlib")
            if not os.path.exists(hive_aux_dir):
                os.mkdir(hive_aux_dir)
                os.chmod(hive_aux_dir, 0o775)

        for d in install_dirs:
            if not os.path.exists(d):
                continue

            for p in glob.glob(os.path.join(d, "juicefs-hadoop*")):
                os.remove(p)
            os.symlink(link, os.path.join(d, "juicefs-hadoop.jar"))

            tools_link = os.path.join(d, "tools.jar")
            if os.path.islink(tools_link) or os.path.exists(tools_link):
                os.remove(tools_link)
            os.symlink(os.path.join(java_home, "lib/tools.jar"), tools_link)

    def _make_cache_dir(self, cache_dirs):
        patterns = cache_dirs.split(':')
        for pattern in patterns:
            dirs = self._expand_dir(pattern)
            for dir in dirs:
                if not os.path.exists(dir):
                    os.makedirs(dir)
                    os.chmod(dir, 0o777)

    def _expand_dir(self, pattern):
        chars = '*?['
        if not any(elem in chars for elem in pattern):
            return [pattern]
        parent, name = os.path.split(pattern)
        rest_name = ""
        while not any(elem in chars for elem in name):
            rest_name = os.path.join(name, rest_name)
            parent, name = os.path.split(parent)

        matched = glob.glob(os.path.join(parent, name))
        return [os.path.join(d, rest_name) for d in matched]

    def configure(self, env):
        import params

        env.set_params(params)


if __name__ == "__main__":
    JuiceFSClient().execute()
